<%#
char *caml_bool(int b) {
    return b ? "true" : "false";
}

int is_littleendian(void) {
    union {
	int i;
	char c[sizeof(int)];
    } v;

    v.i = 1;
    return v.c[0];
}

void try_divmod(int i1, int i2) {
    int d, m;

    d = i1 / i2;
    m = i1 % i2;

    printf("(%d, %d)", d, m);
}
#%>
(* This is an example version for embedded-c demonstration. *)
(* 
   Part of Fail-Safe C Compiler. Produced by Yutaka Oiwa.
   (c) 2001-2005 Yutaka Oiwa.
   (c) 2005-     AIST.

   This file is written by Yutaka Oiwa in 2002 -- 2004. *)
(* <%=s EC_VERSION_MSG %> *)

let bits_of_byte = 8

let sizeof_char = <%=d sizeof(char) %>
let sizeof_short = <%=d sizeof(short) %>
let sizeof_int = <%=d sizeof(int) %>
let sizeof_long = <%=d sizeof(long int) %>
let sizeof_longlong = <%=d sizeof(long long) %>
let sizeof_float = <%=d sizeof(float) %>
let sizeof_double = <%=d sizeof(double) %>
let sizeof_longdouble = <%=d sizeof(long double) %>

let sizeof_pointer = <%=d sizeof(void *) %>

let max_bitfield_width = sizeof_int * bits_of_byte (* 32 *)

let char_is_signed = <%=s caml_bool(0 > (char)-1) %>
let is_littleendian = <%=s caml_bool(is_littleendian()) %>

type quomod_style = 
    Div_round_to_zero | Mod_match_divisor | Mod_always_positive
let quomod_style = 
  match <% try_divmod(5, 3); %>, <% try_divmod(5, -3); %>,
    <% try_divmod(-5, 3); %>, <% try_divmod(-5, -3); %> with
  (* 5/3     5/-3      -5/3      -5/-3 *)
    (1, 2), (-1, 2),  (-1, -2), (1, -2) -> Div_round_to_zero
  | (1, 2), (-2, -1), (-2, 1),  (1, -2) -> Mod_match_divisor
  | (1, 2), (-1, 2),  (-2, 1),  (2, 1) -> Mod_always_positive
  | _ -> failwith "FSC_CONFIG failed: unknown div/mod semantics"

let print_config () = 
  Printf.printf
    "Configuration values:

sizes:
   %d: char
   %d: short
   %d: int
   %d: long
   %d: long long
   %d: float
   %d: double
   %d: long double
   %d: pointers

char is %s.
endianness is %s-endian.
division semantics is \"%s\".
"
    sizeof_char sizeof_short sizeof_int sizeof_long sizeof_longlong
    sizeof_float sizeof_double sizeof_longdouble
    sizeof_pointer
    (if char_is_signed then "signed" else "unsigned")
    (if is_littleendian then "little" else "big")
    (match quomod_style with 
      Div_round_to_zero -> "match with absolute value (normal)"
    | Mod_match_divisor -> "modulo sign matches divisor"
    | Mod_always_positive -> "modulo always positive")

